@@ -8,7 +8,9 @@ from pylint.reporters.json_reporter import JSONReporter
 # GitLab specs:
 #   https://docs.gitlab.com/ee/ci/testing/code_quality.html#implementing-a-custom-tool
 #
-# Output must be info, minor, major, critical, or blocker
+# TODO: Allow a configuration file for severity, categories, etc... (in pyproject.toml?)
+#
+# FIXME: Use a precise map?
 SEVERITY_MAP = {
     "convention": "info",
     "refactor": "minor",
@@ -17,6 +19,18 @@ SEVERITY_MAP = {
     "fatal": "blocker",
 }
 
+# FIXME: Use a precise map?
+# CATEGORIES_MAP = {
+#    "": "Potential bug",
+#    "": "Clarity",
+#    "": "Compatibility",
+#    "": "Complexity",
+#    "": "Duplication",
+#    "": "Performance",
+#    "": "Security",
+#    "": "Style",
+# }
+
 
 def get_fingerprint(item):
     string = ""
@@ -35,16 +49,19 @@ class CodeClimateReporter(JSONReporter):
     extension = "codeclimate"
 
     @staticmethod
-    def serialize(message):
-        item = JSONReporter.serialize(message)
+    def transform(item):
         output = {
-            "fingerprint": get_fingerprint(item),
-            "severity": SEVERITY_MAP[item["type"]],
+            "type": "issue",
+            # (required) check_name: "Bug Risk/Unused Variable",
             "description": item["message"].split("\n")[0],
+            # "content": A markdown snippet describing the issue, including deeper explanations and links to other resources.
+            # (required) "categories": At least one category indicating the nature of the issue being reported.
+            "fingerprint": get_fingerprint(item),
             "location": {"path": item["path"]},
+            "severity": SEVERITY_MAP[item["type"]],
         }
         # Handle lines vs position
-        if item["endLine"] is None:
+        if item.get("endLine") is not None:
             output["location"]["lines"] = {"begin": item["line"]}
         else:
             # FIXME: if begin to ends spans the whole line, might as well use "lines"
@@ -61,6 +78,11 @@ class CodeClimateReporter(JSONReporter):
             }
         return output
 
+    @staticmethod
+    def serialize(message):
+        item = JSONReporter.serialize(message)
+        return CodeClimateReporter.transform(item)
+
 
 def register(linter):
     linter.register_reporter(CodeClimateReporter)
